/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.module.refresh;

import org.apache.log4j.Logger;
import org.builtu.exceptions.FatalError;


/**
 * Base for singleton object that has to be refreshed basing on given refresh strategy 
 * @author borisman
 *
 * @param <T> singleton derived class
 */
public abstract class RefreshSingleton<T> {
	
	private static final Logger logger = Logger.getLogger(RefreshSingleton.class);
	
	private RefreshStrategy strategy;
	private volatile T instance;
	
	public RefreshStrategy getStrategy() {
		return strategy;
	}

	public void setStrategy(RefreshStrategy strategy) {
		this.strategy = strategy;
	}

	public T get() {
		if (instance == null || strategy.hasToBeRefreshed()) {
			synchronized(this) {
				if (instance == null || strategy.hasToBeRefreshed()) {
					final T newT;
					try {
						newT = create();
					} catch (final Throwable t) {
						logger.error(t);
						if (instance == null) throw new FatalError(t);
						return instance;
					}
					instance = newT;
					strategy.refreshed();
				}
			}
		}
		return instance;
	}
	
	protected abstract T create();
}
