/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.markup;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.builtu.collections.graph.NodeImpl;
import org.builtu.util.commutil.StringUtil;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


public class Parser {
	
	private static class ParserHandler extends DefaultHandler {
		public NodeImpl<String> root;
		private StringBuilder text;
		private NodeImpl<String> current;
		private final boolean parseAttributes;
		
		public ParserHandler(final boolean parseAttributes) {
			this.parseAttributes = parseAttributes;
		}

		@Override public void startElement(
				final String uri, 
				final String localName, 
				final String qName,
				final Attributes attributes
		) throws SAXException {
			if (current == null) {
				current = root = new NodeImpl<String>(qName);
			} else {
				final String nodeId = makeUniqueNodeid(current, qName);
				final NodeImpl<String> newNode = new NodeImpl<String>(nodeId);
				current.addChild(newNode);
				current = newNode;
			}
			if (parseAttributes) addAttributes(current, attributes);
		}
		
		@Override public void characters(final char[] ch, final int start, final int length) throws SAXException {
			if (text == null) text = new StringBuilder(length);
			text.append(ch, start, length);
		}

		@Override public void endElement(final String uri, final String localName, final String qName)
			throws SAXException 
		{
			if (text!=null) {
				current.setValue(text.toString());
				text = null;
			}
			current = current.hasParents() ? current.getParentImpl(0) : null;
		}

		private void addAttributes(final NodeImpl<String> node, final Attributes attributes) {
			final NodeImpl<String> attributesNode = new NodeImpl<String>("<attributes>");
			node.addChild(attributesNode);
			for (int i=0; i<attributes.getLength(); i++) {
				final String id = makeUniqueNodeid(attributesNode, attributes.getQName(i));
				final String val = attributes.getValue(i);
				final NodeImpl<String> aNode = new NodeImpl<String>(id);
				aNode.setValue(val);
				attributesNode.addChild(aNode);
			}
		}

		private static String makeUniqueNodeid(final NodeImpl<String> parent, final String nodeId) {
			String id = nodeId;
			int i = 1;
			while (parent.get(id)!=null) {
				i++;
				id = nodeId+"."+i;
			}
			return id;
		}


	}

	public static NodeImpl<String> parseXML(final String markup) 
		throws SAXException, IOException, ParserConfigurationException 
	{
		return parseXML(markup, false);
	}
	
	
	public static NodeImpl<String> parseXML(final String markup, final boolean parseAttributes) 
		throws SAXException, IOException, ParserConfigurationException 
	{
		final InputStream in = new ByteArrayInputStream(markup.getBytes(StringUtil.utf8Charset));
		return parseXML(in, parseAttributes);
	}
	
	public static NodeImpl<String> parseXML(final InputStream in, final boolean parseAttributes) 
		throws SAXException, IOException, ParserConfigurationException 
	{
		final ParserHandler han = new ParserHandler(parseAttributes);
		getParser().parse(in, han);
		if (han.root!=null) {
			return han.root;
		} else {
			return new NodeImpl<String>("");
		}
	}

	
	private static SAXParser getParser() throws ParserConfigurationException, SAXException {
		return getParserFactory().newSAXParser();
	}
	
	private static volatile SAXParserFactory factory;

	private static SAXParserFactory getParserFactory() 
		throws ParserConfigurationException, SAXException 
	{
		if (factory!=null) return factory;
		synchronized(Parser.class) {
			if (factory!=null) return factory;
			factory = SAXParserFactory.newInstance();
		}
		return factory;
	}
	
}
