/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.testing.webmock;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.builtu.util.clown.Xerox;
import org.builtu.util.commutil.Util;


public class Request {
	private String path = "/";
	public final Map<String, String []> params = new HashMap<String, String []>();
	
	public String getPath() {
		return path;
	}
	
	static void checkPath(final String path) {
		if (path.indexOf('?')>=0 || path.indexOf('&')>=0) {
			throw new IllegalArgumentException("path cannot contain characters ? and &, these are the dynamic part"); 
		}
	}

	public void setPath(final String path) {
		checkPath(path);
		this.path = path;
	}

	public void add(final String param, final String value) {
		String [] values = params.get(param);
		if (values == null) {
			values = new String [] {value};
		} else {
			final String [] copy = new String[values.length+1];
			System.arraycopy(values, 0, copy, 0, values.length);
			copy[values.length] = value;
			values = copy;
		}
		params.put(param, values);
	}

	public void parseHttpRequest(final HttpServletRequest request) {
		final Enumeration<?> keys = request.getParameterNames();
		while (keys.hasMoreElements()) {
			final String key = (String) keys.nextElement();
			final String[] values = request.getParameterValues(key);
			params.put(key, values);
		}
	}

	
	public void set(final String param, final String ... values) {
		params.put(param, values);
	}
	
	public Request deepClone() {
		return Xerox.deepCloneFast(this);
	}

	@Override public int hashCode() {
		int res = path.hashCode();
		for (final Map.Entry<String, String[]> entry : params.entrySet()) {
			res += entry.getKey().hashCode();
			/*for (final String v : entry.getValue()) {
				res += v.hashCode();
			}*/
		}
		return res;
	}
	
	@Override public boolean equals(final Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof Request)) return false;
		final Request rhs = (Request) obj;
		
		if (!path.equals(rhs.path)) return false;

		final Set<String> thisKeys = params.keySet();
		final Set<String> rhsKeys = rhs.params.keySet();
		
		if (thisKeys.size()!=rhsKeys.size()) return false;
		
		for (final String key : thisKeys) {
			final String[] thisValArray = params.get(key);
			final String[] rhsValArray = rhs.params.get(key);
			
			if (thisValArray.length == 0 || rhsValArray.length == 0) continue;
			
			if (thisValArray.length != rhsValArray.length) return false;
			
			final List<String> rhsVal = copy(rhsValArray);

			for (final String v : thisValArray) {
				if (!rhsVal.remove(v)) return false;
			}
		}
		
		return true;
	}
	
	@Override public String toString() {
		final StringBuilder p = new StringBuilder(params.size()<<6);
		for (final Map.Entry<String, String[]> entry : params.entrySet()) {
			if (p.length()>0) p.append('&');
			final String [] values = entry.getValue();
			final String keyEncoded = Util.urlencode(entry.getKey()); 
			if (values.length == 0) {
				p.append(keyEncoded);
			} else {
				for (final String v : entry.getValue()) {
					p.append(keyEncoded);
					p.append('=');
					p.append(Util.urlencode(v));
				}
			}
		}
		
		final StringBuilder res = new StringBuilder(p.length()+50);
		res.append("Request [path=" + path);
		if (p.length()>0) {
			res.append(", params:");
			res.append(p);
		}
		res.append(']');
		return res.toString();
	}

	private static List<String> copy(final String[] arr) {
		final List<String> res = new ArrayList<String>(arr.length);
		for (final String s : arr) res.add(s);
		return res;
	}


}
