/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.commutil;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class DateUtil {
	
	public static final Date day(final int year, final int month, final int day) {
		if (year < 0) throw new IllegalArgumentException("year should be > 0 (got "+year+")");
		if (month < 1 || month > 12) throw new IllegalArgumentException("month should be 1-12 (got "+month+")");
		if (day < 1 || day > 31) throw new IllegalArgumentException("day should be 1-31 (got "+day+")");
		final Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.YEAR, year);
		calendar.set(Calendar.MONTH, month-1);
		calendar.set(Calendar.DAY_OF_MONTH, day);
		truncateToDay(calendar);
		return calendar.getTime();
	}

	public static Date truncateToDay(final Date date) {
		final Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		truncateToDay(calendar);
		return calendar.getTime();
	}
	
	public static void truncateToDay(final Calendar calendar) {
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
	}
	
	public static final int HOUR_MULTIPLIER = 60 * 60 * 1000;
	
	public static long addHour(final int hour) {
		return hour * HOUR_MULTIPLIER;
	}
	
	public static final int DAY_MULTIPLIER = 24 * 60 * 60 * 1000;
	
	public static long addDay(final int day) {
		return day * DAY_MULTIPLIER;
	}
	
	public static Date addDays(final Date d, final int howMany) {
		final Calendar cal = Calendar.getInstance();
		cal.setTime(d);
		cal.add(Calendar.DAY_OF_MONTH, howMany);
		return cal.getTime();
	}

	public static Date addMonths(final Date d, final int howMany) {
		final Calendar cal = Calendar.getInstance();
		cal.setTime(d);
		cal.add(Calendar.MONTH, howMany);
		return cal.getTime();
	}

	public static Date addHours(final Date d, final int howMany) {
		final Calendar cal = Calendar.getInstance();
		cal.setTime(d);
		cal.add(Calendar.HOUR, howMany);
		return cal.getTime();
	}

	public static enum Precision {
		Millisecond("yyyy-MM-dd HHmm.SSS"),
		Minute("yyyy-MM-dd HHmm"),
		Hour("yyyy-MM-dd HH"),
		Day("yyyy-MM-dd");
		
		public final String formatString;
		public final SimpleDateFormat format;
		
		private Precision(final String formatString) {
			this.formatString = formatString;
			this.format = new SimpleDateFormat(formatString);
		}
		
	}
	
	/**
	 * Format using millisecond precision (yyyy-MM-dd HHmm.SSS), example: 2010-03-28 1243.123
	 */
	public static String format(final Date date) {
		return format(date, Precision.Millisecond);
	}

	/**
	 * Format date using specified precision (see DateUtil.Precision)
	 */
	public static String format(final Date date, final Precision precision) {
		return precision.format.format(date);
	}
	
	/**
	 * Parse date in format yyyy-MM-dd HHmm.SSS, hours, minutes, milliseconds may be ommitted (they will be zeroed 
	 * then), for example:
	 * <code>
	 * DateUtil.parse("2010-03-28")
	 * DateUtil.parse("2010-03-28 00")
	 * DateUtil.parse("2010-03-28 0000")
	 * Dateutil.parse("2010-03-28 0000.000")
	 * <code>
	 * will give the very same result
	 */
	public static Date parse(final String string) throws ParseException {
		return parse(string, null);
	}
	
	/**
	 * Parse date string with given precision. Note that if you provide string with broader precision than precision
	 * specified, the ParseException will be thrown, for example:
	 * <code>
	 * DateUtil.parse("2009-02-04 12", DateUtil.Precision.Day) 
	 * </code>
	 * will throw an exception.
	 * 
	 * You can pass null as precision, and if you do so, the method will determine the precision by scanning the
	 * string, so for example:
	 * <code>
	 * DateUtil.parse("2010-03-28", null)
	 * DateUtil.parse("2010-03-28 00", null)
	 * DateUtil.parse("2010-03-28 0000", null)
	 * Dateutil.parse("2010-03-28 0000.000", null)
	 * <code>
	 * will give the very same result
	 */
	public static Date parse(final String string, Precision precision) throws ParseException {
		if (precision == null) {
			for (final Precision p : Precision.values()) {
				if (p.formatString.length() == string.length()) {
					precision = p;
					break;
				}
			}
			if (precision == null) {
				throw new ParseException(string, 0);
			}
		} else if (string.length()!=precision.formatString.length()) {
			throw new ParseException(string, 0);
		}
		return precision.format.parse(string);	
	}

}
