/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.commutil;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.io.Reader;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import org.builtu.exceptions.FatalError;


public class IOUtil {

	/**
	 * Stream bytes from in to out
	 */
	public static void stream(final InputStream in, final OutputStream out) throws IOException {
		final byte [] buffer = new byte[4096];
		int read = in.read(buffer);
		while (read>=0) {
			if (read>0) out.write(buffer, 0, read);
			read = in.read(buffer);
		}
	}

	public static void close(OutputStream in) {
        if (in != null) {
            try {
                in.close();
            } catch (IOException e) {
            	throw new FatalError(e);
            }
        }
    }

    public static void close(Reader in) {
        if (in != null) {
            try {
                in.close();
            } catch (IOException e) {
            	throw new FatalError(e);
            }
        }
    }

    public static void close(Writer in) {
        if (in != null) {
            try {
                in.close();
            } catch (IOException e) {
            	throw new FatalError(e);
            }
        }
    }

    public static void close(InputStream in) {
        if (in != null) {
            try {
                in.close();
            } catch (IOException e) {
            	throw new FatalError(e);
            }
        }
    }
	
	public static byte[] readRaw(final InputStream in) throws IOException {
		final ByteArrayOutputStream res = new ByteArrayOutputStream ();
		final byte [] buffer = new byte[4096];
		int read = in.read(buffer);
		while (read >= 0) {
			if (read>0) res.write(buffer, 0, read);
			read = in.read(buffer);
		}
		return res.toByteArray();
	}


	public static byte[] readRaw(final File source) throws IOException {
		final byte [] res = new byte [(int)source.length()];
		final InputStream in = new FileInputStream(source);
		try {
			final int read = in.read(res);
			if (read != res.length) throw new IOException("Read "+read+" bytes, expected to read="+res.length);
			if (in.read() != -1) throw new IOException("More data available when all data were read");
		} finally {
			in.close();
		}
		return res;
	}

	public static void writeRaw(final File file, final byte [] contents) 
		throws IOException 
	{
		final OutputStream out = new FileOutputStream(file);
		try {
			out.write(contents);
		} finally {
			out.close();
		}
	}

	public static String readContents(final InputStream in, final Charset charset, int lengthHint)
		throws IOException
	{
		if (lengthHint<=0) lengthHint = 4096;
		final byte [] buffer = new byte[lengthHint];
		final StringBuilder res = new StringBuilder(lengthHint);
		int read = in.read(buffer);
		while (read >= 0) {
			final String part = new String(buffer, 0, read, charset);
			res.append(part);
			read = in.read(buffer);
		}
		return res.toString();
	
	}
	
	public static String readContents(final InputStream in, final Charset charset)
		throws IOException 
	{
		return readContents(in, charset, 0);
	}
	
	public static String readUTF8Contents(final InputStream in)
		throws IOException 
	{
		return readContents(in, StringUtil.utf8Charset, 0);
	}
	
	public static List<String> readUTF8Strings(final File file) throws IOException {
		final InputStream in = new FileInputStream(file);
		try {
			return readUTF8Strings(in);
		} finally {
			in.close();
		}
	}
	
	public static String readFileContents(final File file, final Charset charset) throws IOException {
		final long lLength = file.length();
		if (lLength>Integer.MAX_VALUE) throw new FatalError("File is too big: "+lLength+" bytes");
		final InputStream in = new FileInputStream(file);
		try {
			return readContents(in, charset, (int)lLength);
		} finally {
			in.close();
		}
	}
	
	public static void writeFileContents(final File file, final String contents, final Charset charset) 
		throws IOException 
	{
		final OutputStream out = new FileOutputStream(file);
		try {
			final OutputStreamWriter w = new OutputStreamWriter(out, charset);
			w.write(contents);
			w.flush();
		} finally {
			out.close();
		}
	}

	public static String readASCIIFileContents(final File file) throws IOException {
		return readFileContents(file, StringUtil.asciiCharset);
	}

	public static void writeASCIIFileContents(final File file, final String contents) 
		throws IOException
	{
		writeFileContents(file, contents, StringUtil.asciiCharset);
	}
	
	public static String readUTF8FileContents(final File file) throws IOException {
		return readFileContents(file, StringUtil.utf8Charset);
	}
	
	public static void writeUTF8FileContents(final File file, final String contents) 
		throws IOException
	{
		writeFileContents(file, contents, StringUtil.utf8Charset);
	}
	
	public static List<String> readASCIIStrings(final File file) throws IOException {
		final List<String> res = new ArrayList<String>((int)file.length()>>5);
		final RandomAccessFile ra = new RandomAccessFile(file, "r");
		try {
			String line = ra.readLine();
			while (line != null) {
				res.add(line);
				line = ra.readLine();
			}
		} finally {
			ra.close();
		}
		return res;
	}
    
	public static void writeASCIIStrings(final File file, final Iterable<String> strings) throws IOException {
		writeStrings(file, strings, StringUtil.asciiCharset);
	}

	public static void writeUTF8Strings(final File file, final Iterable<String> strings) throws IOException {
		writeStrings(file, strings, StringUtil.utf8Charset);
	}

	public static void writeStrings(final File file, final Iterable<String> strings, final Charset charset) 
		throws IOException 
	{
		final OutputStream out = new FileOutputStream(file);
		try {
			final OutputStreamWriter w = new OutputStreamWriter(out, charset);
			
			boolean first = true;
			for (final String s : strings) {
				if (first) {
					first = false;
				} else {
					w.write('\n');
				}
				w.write(s);
			}
			
			w.flush();
		} finally {
			out.close();
		}
	}	
	
	public static List<String> readUTF8Strings(final InputStream in)
	{
		return readStrings(in, StringUtil.utf8Charset);
	}

	public static List<String> readASCIIStrings(final InputStream in)
	{
		return readStrings(in, StringUtil.asciiCharset);
	}
	
	public static List<String> readStrings(final InputStream in, final Charset charset)
	{
		final List<String> res = new ArrayList<String>(); 
	    final BufferedReader br = new BufferedReader(new InputStreamReader(in, charset));
	    try {
	        String line = br.readLine();
	        while (line != null) {
	            res.add(line);
	            line = br.readLine();
	        }
	    } catch (final IOException e) {
	        throw new FatalError(e);
	    } finally {
	        try {
	            in.close();
	        } catch (final IOException e) {
	            throw new FatalError(e);
	        }
	    }
	    return res;
	}

}
